/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.naming;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.naming.Reference;

import org.exoplatform.container.component.ComponentPlugin;
import org.exoplatform.container.configuration.ConfigurationException;
import org.exoplatform.container.xml.InitParams;
import org.exoplatform.container.xml.ValueParam;
import org.picocontainer.Startable;

/**
 * Created by The eXo Platform SARL        .
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov</a>
 * @version $Id: $
 */

public class InitialContextInitializer implements Startable {

  private List <BindReferencePlugin> bindReferencesPlugins;
  
  private String defaultContextFactory; 
  
  public InitialContextInitializer(InitParams params) throws NamingException, 
  ConfigurationException {
    
    ValueParam defFactoryParam = params.getValueParam("default-context-factory");
    if(defFactoryParam == null)
      throw new ConfigurationException("No 'default-context-factory' parameter");
    defaultContextFactory = defFactoryParam.getValue();

    if(System.getProperty(Context.INITIAL_CONTEXT_FACTORY) == null)
      System.setProperty(Context.INITIAL_CONTEXT_FACTORY , defaultContextFactory);
    
    
    bindReferencesPlugins = new ArrayList<BindReferencePlugin>();
  }
  
  public void start() {
    try {
      Context initialContext = new InitialContext();
      for (int i = 0; i < bindReferencesPlugins.size(); i++) {
        BindReferencePlugin plugin = bindReferencesPlugins.get(i);
        initialContext.rebind(plugin.getBindName(), plugin.getReference());
      }
    } catch (NamingException e) {
      e.printStackTrace();
    }
  }

  public void stop() {
  }

  
  // for out-of-container testing
  private InitialContextInitializer(String name, Reference reference) throws NamingException {
    System.setProperty(Context.INITIAL_CONTEXT_FACTORY,
      "org.exoplatform.services.jcr.impl.mock.SimpleContextFactory");
    Context initialContext = new InitialContext();
    initialContext = new InitialContext();
    initialContext.rebind(name, reference);
  }
  
  public void addPlugin(ComponentPlugin plugin) {
    if (plugin instanceof BindReferencePlugin)
      bindReferencesPlugins.add((BindReferencePlugin)plugin);
  }

  public ComponentPlugin removePlugin(String name) {
    return null;
  }

  public Collection getPlugins() {
    return bindReferencesPlugins;
  }

  // for out-of-container testing
  public static void initialize(String name, Reference reference) throws NamingException {
    new InitialContextInitializer(name, reference);
  }

  public String getDefaultContextFactory() {
    return defaultContextFactory;
  }

  
}
