/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.database.jdbc;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import org.apache.commons.logging.Log;
import org.exoplatform.container.ExoContainer;
import org.exoplatform.container.component.ComponentLifecycle;
import org.exoplatform.container.component.ComponentPlugin;
import org.exoplatform.services.log.ExoLogger;
import org.exoplatform.services.naming.InitialContextInitializer;

/**
 * Created by The eXo Platform SARL        .
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov</a>
 * @version $Id: DBSchemaCreator.java 5150 2006-04-17 06:44:21Z geaz $
 */

public class DBSchemaCreator implements ComponentLifecycle {
  
  private static Log log = ExoLogger.getLogger("database.DBSchemaCreator");
  
  private List <CreateDBSchemaPlugin> createDBSchemaPlugins = new ArrayList<CreateDBSchemaPlugin>();
  
  public DBSchemaCreator(InitialContextInitializer contextInit) {

  }
  
//  public void start() {
//    for (int i = 0; i < createDBSchemaPlugins.size(); i++) {
//      CreateDBSchemaPlugin plugin = createDBSchemaPlugins.get(i);
//      try {
//        createTables(plugin.getDataSource(), plugin.getScript());
//      } catch (NamingException e) {
//        e.printStackTrace();
//      } catch (SQLException e) {
//        e.printStackTrace();
//      }
//    }
//  }
//
//  public void stop() {
//  }
  
  
  // for testing only
  private DBSchemaCreator(String dsName, String script) throws SQLException, NamingException {
    createTables(dsName, script);
  }

  public void createTables(String dsName, String script) throws NamingException, SQLException {
    InitialContext context = new InitialContext();
    DataSource ds = (DataSource)context.lookup(dsName);
    Connection conn = ds.getConnection();
    conn.setAutoCommit(false);
    try {
      log.debug("Execute script: \n"+script);
      conn.createStatement().executeUpdate(script);
      conn.commit();
    } catch (SQLException e) {
      conn.rollback();
      //throw e;
      log.error("Could not create db schema "+script+" Reason: "+e);
    } finally {
      conn.close();
    }
    log.info("DB schema created succesfully.");
  }
  
  public void addPlugin(ComponentPlugin plugin) {
    if (plugin instanceof CreateDBSchemaPlugin)
      createDBSchemaPlugins.add((CreateDBSchemaPlugin)plugin);
  }

  public ComponentPlugin removePlugin(String name) {
    return null;
  }

  public Collection getPlugins() {
    return createDBSchemaPlugins;
  }
  
  // for testing
  public static void initialize(String dsName, String script) throws SQLException, NamingException {
    new DBSchemaCreator(dsName, script);
  }

  public void destroyComponent(ExoContainer container) throws Exception {
  }

  public void initComponent(ExoContainer container) throws Exception {
    for (int i = 0; i < createDBSchemaPlugins.size(); i++) {
      CreateDBSchemaPlugin plugin = createDBSchemaPlugins.get(i);
      try {
        createTables(plugin.getDataSource(), plugin.getScript());
      } catch (NamingException e) {
        e.printStackTrace();
      } catch (SQLException e) {
        e.printStackTrace();
      }
    }
  }

  public void startComponent(ExoContainer container) throws Exception {
  }

  public void stopComponent(ExoContainer container) throws Exception {
  }


}
