/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.ticket;

import java.util.Date;

/**
 * Created by The eXo Platform SARL
 * Author : Chung Nguyen
 *          nguyenchung136@yahoo.com
 * Jan 5, 2006
 * 
 * This class is used  to control the security of  a task or a resource. 
 */
public interface Ticket {
  static final  public int UNLIMITED_ACCESS =  -1 ;
  
  static final  public int VALID_STATUS =  0 ;
  static final  public int EXPIRED_STATUS =  1 ;
  static final  public int LOCKED_STATUS =  2 ;
  /**
   * this method return the id of ticket
   * @return : the id of the ticket
   */
  public String getId() ;
  /**
   * @return : The number that method TicketService.executeTicketCommand(..) is called for a ticket
   */
  public int  getAccessCounter() ;
  /**
   * @return : The max  number of time that the TicketCommand can call execute()
   */
  public int  getLimitAccess() ;
  /**
   * @return : Name of the creator Ticket
   */
  public String getCreator() ;
  /**
   * @return : The time that the ticket is created
   */
  public Date getCreationTime() ;
  /**
   * @return The username of the last user who invoke the method TicketService.executeTicketCommand(..).
   * Return null is the user is not identified
   * 
   */
  public String getLastAccessUser() ;
  /**
   * @return : The last time that the method TicketService.executeTicketCommand(..) is invoked
   */
  public Date getLastAccessTime() ;
  /**
   * @return : the amount of  time that the ticket should last in milisecond 
   */
  public long getLiveTime() ;
  /**
   * @return : The list of the users who can access  the ticket
   */
  public String[] getAlowUsers();
  /**
   * @return : The list of the membership that the user  should have at least one membership
   * to have the right to access the ticket.  The membership  should have the form : 
   * membersipName:/group/name. This "membershipName:/group/name mean the access user has to 
   * have  a membership with the 'membershipName' in the  group '/group/name'
   * 
   * TODO  this method should be rename to  getRequiredMemberships()
   */
  public String[] getRequiredPermissions();
  /**
   * @return : The status of the ticket
   */
  public int getStatus() ;
}

