/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.ticket;

/**
 * Created by The eXo Platform SARL
 * Author : Chung Nguyen
 *          chungnv@users.sourceforge.net
 * Jan 5, 2006
 * 
 * This service is used to manage the  the ticket , a ticket is an instance of a security control
 * for a task or a resource. A ticket 
 */
public interface TicketService {
  /**
   * This method  should create a ticket object and store the ticket in the database.
   * 
   * @param liveTime:   the amount of  time that the ticket should last in milisecond 
   * @param limitAccess: The max  number of time that the TicketCommand can call execute()
   * @param owenCreator: Creator  
   * @param allowedUser: The list of the users who can access  the ticket
   * @param requiredPermissions
   * @param command : Object control action of one Ticket
   * @return The  id  of the ticket
   * @throws Exception
   */
  public String createTicket(long liveTime , int limitAccess,String desc, String owenCreator,
                             String[] allowedUser, String[] requiredPermissions, 
                             TicketCommand command) throws Exception ;
  /**
   * the method should create a ticket object and store the ticket in the database
   * @param liveTime: the amount of  time that the ticket should last in milisecond
   * @param limitAccess: the max number of time that the TicketComand can call execute();
   * @param command: Object control action of one Ticket 
   * @return the id of the ticket
   * @throws Exception
   */
  public String createTicket(long liveTime , int limitAccess,String desc,TicketCommand command) throws Exception ;
  /**
   * the method should return a ticket object with  that match the id or null
   * @param id: the id of the ticket 
   * @return the ticket
   * @throws Exception
   */
  public Ticket  getTicket(String id) throws Exception ;
  /** 
   * the method should return a ticket object with  id and username of the ticket correlative
   * the method should return a ticket object with  id of the ticket correlative
   * @param id: the id of the ticket
   * @param accessUser: username of the ticket
   * @return the ticket
   * @throws Exception Usually the exception is throwed when the method fail to access the 
   * database
   * 
   * TODO  Remove this method as read the information of a ticket doesn't need the security 
   * check
   */
  public Ticket  getTicket(String id, String accessUser) throws Exception ;
  /**
   * This method  should search the ticket in its database, load the ticket command
   * and then invoke the method TicketCommand.execute(). This method should check the 
   * the security according to  the accessUser and permissions of the ticket  before
   * it invoke the execute method 
   * @param ticketId: the id of the ticket
   * @return : The result of TicketCommand.execute()  of the ticket 
   * @throws Exception
   */
  public Object  executeTicketCommand(String ticketId) throws Exception ;
  /**
   * the methode should  load the ticket command and then invoke the method 
   * TicketCommand.execute(). This method should check the the security according 
   * to  the accessUser and permissions of the ticket  before it invoke the execute method 
   * @param ticket: the ticket
   * @return: Object of the ticketcomand
   * @throws Exception
   */
  public Object  executeTicketCommand(Ticket ticket) throws Exception ;
  /**
   * This method does the same thing in  executeTicketCommand(String ticketId) but in this 
   * case the user  who access the ticket command is identified.
   * @param ticketId : the id of the ticket
   * @param accessUser : the user  who try to access the ticket
   * @return :  the result of the TicketCommand.execute()
   * @throws Exception
   */
  public Object  executeTicketCommand(String ticketId, String accessUser) throws Exception ;
  /**
   * This method does the same thing in  executeTicketCommand(Ticket ticket) but in this 
   * case the user  who access the ticket command is identified.
   * @param ticketId : the id of the ticket
   * @param accessUser : the user  who try to access the ticket
   * @return :  the result of the TicketCommand.execute()
   * @throws Exception
   */
  public Object  executeTicketCommand(Ticket ticket, String accessUser) throws Exception ;
  
  /**
   * This method  should remove the ticket  from the its database
   * @param id 
   * @throws Exception
   */
  public void removeTicket(String id) throws Exception;
}