/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.ticket.impl;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;


import org.exoplatform.commons.utils.ExoProperties;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.ticket.Ticket;
import org.exoplatform.services.ticket.TicketCommand;
import org.exoplatform.services.ticket.TicketService;

/**
 * Created by The eXo Platform SARL
 * Author : Chung Nguyen
 *          nguyenchung136@yahoo.com
 * Jan 5, 2006
 */
public class TicketServiceImpl implements TicketService {
  private HibernateService hservice_ ;
  private Map<String, String> locks_ = new HashMap<String, String>() ;
  
  public TicketServiceImpl(HibernateService hservice) {
    hservice_  = hservice ;
  }
  
  public String createTicket(long liveTime, int limit,String desc,TicketCommand command) throws Exception {
    return createTicket(liveTime, limit,desc,null, null, null, command) ;
  }
  
  public String createTicket(long liveTime, int limitAccess,String desc,String owenCreator, 
                             String[] allowedUser, String[] requiredPermissions, 
                             TicketCommand command) throws Exception {
    TicketImpl ticket=new TicketImpl();
    ticket.setId(Integer.toString(ticket.hashCode()));
    ticket.setCreationTime(new Date());
    ticket.setAccessCounter(0);
    ticket.setLimitAccess(limitAccess);
    ticket.setCreator(owenCreator);
    ticket.setLiveTime(liveTime);
    ticket.setDescription(desc);
    
    if(limitAccess == -1){
      ticket.setStatus(Ticket.UNLIMITED_ACCESS);
    }else if(limitAccess <-1 || limitAccess == 0){
      ticket.setStatus(Ticket.LOCKED_STATUS);
    }else ticket.setStatus(Ticket.VALID_STATUS);
    
        
    ticket.setAllowedUsers(allowedUser) ;
    ticket.setRequiredPermissions(requiredPermissions);
    ticket.setTicketCommandClass(command.getClass().getName()) ;
    
   // Map<String, String> props = command.getProperties() ;
    //if(props != null && props.size() > 0) {
      ticket.setProperties(command.getProperty("param1")) ;
    //} 
    hservice_.save(ticket);
    return ticket.getId();
  }
  /**The methode get one ticket with the id of ticket
   * id : the id of the ticket
   * return : the ticket
   */
  public Ticket getTicket(String id) throws Exception {
    TicketImpl impl =  (TicketImpl) hservice_.findOne(TicketImpl.class, id) ;
    return impl;
  }
  /** The method get one ticket with the id of ticket and Access Users
   *  id : The id of ticket
   *  accessUser : User of the ticket
   *  return : the ticket 
   */
  public Ticket getTicket(String id, String accessUser) throws Exception {
    boolean checkAcessUser=false;
    
    if(accessUser==null) return null;
    TicketImpl impl =  (TicketImpl)getTicket(id) ;
    if(impl==null) return null;
    
    String[] users=impl.getAlowUsers();
    for(int i=0;i<users.length;i++){
      if(users[i].equals(accessUser)){
        checkAcessUser=true;
        break;
      }
    }
    if(checkAcessUser){
       return impl;
    }
    return null;
  }
  
  /**
   * return Array Required Permissions of TicketCommand 
   */
  public Object executeTicketCommand(String ticketId) throws Exception {
      TicketImpl ticket=(TicketImpl) getTicket(ticketId);
      return executeTicketCommand(ticket, null) ;
  }
  /*  
   *  Ticket ticket 
   * @see org.exoplatform.services.ticket.TicketService#executeTicketCommand(org.exoplatform.services.ticket.Ticket)
   */
  public Object executeTicketCommand(Ticket ticket) throws Exception {
    return this.executeTicketCommand(ticket,null);
  }
  
  /*  
   * @see org.exoplatform.services.ticket.TicketService#executeTicketCommand(java.lang.String, java.lang.String)
   */
  public Object executeTicketCommand(String ticketId, String accessUser) throws Exception {
    TicketImpl ticket = (TicketImpl) getTicket(ticketId,accessUser);
    return executeTicketCommand(ticket, accessUser) ;
  }
  /*  
   * @see org.exoplatform.services.ticket.TicketService#executeTicketCommand(org.exoplatform.services.ticket.Ticket, java.lang.String)
   */
  public Object executeTicketCommand(Ticket ticket, String accessUser) throws Exception {
    TicketImpl ticketImpl = null;
    try{
      lock(ticket.getId());
      ticketImpl=(TicketImpl)ticket;
      lockService(ticket);
      String[] users= ticketImpl.getAlowUsers() ;
      boolean allow = false ;
      int status = 0; 
      int count = 0;
      if(users == null) {
        // set User Access
        status = ticketImpl.getStatus() ;
        count = ticketImpl.getAccessCounter() ;
        if(status == Ticket.VALID_STATUS || status == Ticket.UNLIMITED_ACCESS){
          ticketImpl.setLastAccessUser(null);
          
          ClassLoader cl = Thread.currentThread().getContextClassLoader() ;
          Class clazz = cl.loadClass(ticketImpl.getTicketCommandClass()) ;
          TicketCommand command = (TicketCommand)clazz.newInstance() ;
          String properties = ticketImpl.getProperties() ;
          if(properties != null && properties.length() > 0) {
            ExoProperties  props = new ExoProperties() ;
            props.addPropertiesFromText(properties) ;
            command.setProperties(props) ;
          }
          ticketImpl.setTicketCommand(command) ;
       
        }else{
          ticketImpl.setAccessCounter(count);
          hservice_.save(ticketImpl);
          ticketImpl = null;
        }
        
        allow = true ;
      }  else {
        for(int i=0;i<users.length;i++){
          if(users[i].equals(accessUser)){
            //set User Access
            if(ticketImpl.getStatus() == Ticket.VALID_STATUS){
              ticketImpl.setLastAccessUser(accessUser);
              
              ClassLoader cl = Thread.currentThread().getContextClassLoader() ;
              Class clazz = cl.loadClass(ticketImpl.getTicketCommandClass()) ;
              TicketCommand command = (TicketCommand)clazz.newInstance() ;
              String properties = ticketImpl.getProperties() ;
              if(properties != null && properties.length() > 0) {
                ExoProperties  props = new ExoProperties() ;
                props.addPropertiesFromText(properties) ;
                command.setProperties(props) ;
              }
              ticketImpl.setTicketCommand(command) ;
              
            }            
            allow = true ;
            break;
          }
        }
      }
      
      if(!allow) {
        ticketImpl.setAccessCounter(count-1);
        hservice_.save(ticketImpl);
        throw new Exception("The user '" + accessUser + "' do now have the permission to access the ticket") ;
      }
      //should check the permissions here if the required permissions is not null.
      if (ticketImpl == null) return Integer.toString(status) ;
      
      TicketCommand command = ticketImpl.getTicketCommand();
      if (command == null)    return Integer.toString(status);
      
      return command.execute();
      // update the ticket data ;
    } catch (Exception e) {
      throw e;
    }finally{
      unlock(ticket.getId());
      if(ticketImpl != null&&ticketImpl.getAccessCounter() == ticketImpl.getLimitAccess()){
        ticketImpl.setStatus(Ticket.LOCKED_STATUS);
        hservice_.save(ticketImpl);
      }
     
    }
  }
  /**
   * Return AccessCount employ Services after Increase AccessCount  
   * If AccessCount = LimitAccess , don't  Increase AccessCount and Lock Service
   **/
  private int increaseCount(TicketImpl ticketImpl){
    
    int nowAccessCount= ticketImpl.getAccessCounter();
    if(ticketImpl.getLimitAccess() == Ticket.UNLIMITED_ACCESS){
      nowAccessCount++;
      return nowAccessCount;
    }
    if(nowAccessCount == ticketImpl.getLimitAccess()) {
      ticketImpl.setStatus(Ticket.LOCKED_STATUS);
      try {
        hservice_.save(ticketImpl);
      } catch (Exception e) {
        e.printStackTrace();
      }
    }
    if(nowAccessCount < ticketImpl.getLimitAccess()) nowAccessCount++;
    return nowAccessCount;
  }
  
  /**
   * Method control Lock Setvice of Ticket 
   * when nowAccessCount=LimitAccess Or nowTime > lockTime
   */
  private void lockService(Ticket ticket){
    TicketImpl ticketImpl=(TicketImpl)ticket;
    int status = ticketImpl.getStatus() ;
    if(status == Ticket.VALID_STATUS  || status == Ticket.UNLIMITED_ACCESS ){
      Long timeLock=ticketImpl.getLastAccessTime().getTime();
      if(new Date().getTime()>timeLock){
        ticketImpl.setStatus(Ticket.EXPIRED_STATUS);
      }else{
        int nowAccessCount=increaseCount(ticketImpl);
        ticketImpl.setAccessCounter(nowAccessCount);
      }
      // Update the ticket data
      try {
        hservice_.save(ticketImpl);
      } catch (Exception e) {
        e.printStackTrace();
      }
    }
  }
  
  synchronized private void lock(String ticketId) throws Exception {
    if(locks_.get(ticketId) !=  null ) {
      throw new Exception("Some one is using the  ticket") ;
    }
    locks_.put(ticketId, ticketId) ;
  }
  
  synchronized private void unlock(String ticketId) throws Exception {
    locks_.remove(ticketId) ;
  }
  public void removeTicket(String ticketId) throws Exception {
    Ticket ticket=getTicket(ticketId);
    hservice_.remove(ticket);
  }

}