/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.faces.core.component;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.List;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.validator.Validator;

import org.apache.commons.fileupload.FileItem;
/**
 * Wed, Dec 22, 2003 @ 23:14
 * @author: Tuan Nguyen
 * @email: tuan08@users.sourceforge.net
 * @version: $Id: UIDateInput.java,v 1.2 2004/10/16 21:13:50 tuan08 Exp $
 */
public class UIDateInput extends UIInput {
  public static final int TEXT = 0;
  public static final int PASSWORD = 1;  

  public static final String DAY_EXTENSION = "_dd";
  public static final String MONTH_EXTENSION = "_mm";
  public static final String YEAR_EXTENSION = "_yyyy";  
  
  protected GregorianCalendar calendar_ ;
  private int minYearRange_ = 2000 ;
  private int maxYearRange_ = 2010 ;

  public UIDateInput(String name, Date date) {
  	  setId(name) ;
    name_ = name ;
    calendar_ = new GregorianCalendar() ;
    calendar_.set(Calendar.HOUR_OF_DAY, 23) ;
    calendar_.set(Calendar.MINUTE, 59) ;
    maxYearRange_ = calendar_.get(Calendar.YEAR) ;
    calendar_.setTime(date);
    editable_ = true ;
    setRendererType("DateInputRenderer") ;
  }    
  
  public String getFamily() {  return COMPONENT_FAMILY ; }
    
  public Object getDecodedValue() { return calendar_.getTime() ; }
  
  public Calendar getCalendar() { return calendar_ ; }
  
  public int getMinYearRange() { return minYearRange_ ; }
  public void setMinYearRange(int year) { minYearRange_ = year ; }
  
  public int getMaxYearRange() { return maxYearRange_ ; }
  public void setMaxYearRange(int year) { maxYearRange_ = year ; }
  
  public UIDateInput addValidator(Validator validator) {
    addComponentValidator(validator) ;
    return this ;
  }
  
  public UIDateInput addValidator(Class clazz) {
    addComponentValidator(clazz) ;
    return this ;
  }
  
  public UIDateInput addDate(Date date) {
    calendar_.setTime(date);
    return this ;
  }
  
  final public  void processValidators(FacesContext context) {
    processValidators(context, calendar_) ;
  }
  
  public void decodeFromMultipartFields(FacesContext context,
      UIComponent component, List items) {
    String dayValue = "";
    String monthValue = "";
    String yearValue = "";
    Iterator iter = items.iterator();
    while (iter.hasNext()) {
      FileItem item = (FileItem) iter.next();
      if (item.isFormField()) {
        String name = item.getFieldName();      
        if(name.equals(getName() + DAY_EXTENSION))
          dayValue = item.getString();
        else if(name.equals(getName() + MONTH_EXTENSION))
          monthValue = item.getString();
        else if(name.equals(getName() + YEAR_EXTENSION))
          yearValue = item.getString();
      }          
    }
    try {
      Calendar cal = getCalendar();
      cal.set(Calendar.DAY_OF_MONTH, Integer.parseInt(dayValue));
      cal.set(Calendar.MONTH, Integer.parseInt(monthValue));
      cal.set(Calendar.YEAR, Integer.parseInt(yearValue));
    } catch (NumberFormatException ex) {      
    }    
  }
  
}