/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.faces.core.component;

import java.util.Iterator;
import java.util.List;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.validator.Validator;

import org.apache.commons.fileupload.FileItem;
import org.exoplatform.faces.FacesUtil;

/**
 * Wed, Dec 22, 2003 @ 23:14
 * @author: Tuan Nguyen
 * @email: tuan08@users.sourceforge.net
 * @version: $Id: UIStringInput.java,v 1.11 2004/08/18 02:37:24 tuan08 Exp $
 */
public class UIStringInput extends UIInput {
  public static final String COMPONENT_FAMILY = "org.exoplatform.faces.core.component.UIStringInput";

  public static final int TEXT = 0;
  public static final int PASSWORD = 1;

  protected String title_;
  protected int type = 0;

  public UIStringInput(String name, String text) {
    setId(name);
    name_ = name;
    value_ = text;
    scripting_ = null;
    editable_ = true;
    setRendererType("StringInputRenderer");
    FacesUtil.createValueBinding(getFacesContext(), this, "text", text);
  }

  public UIStringInput(String name, String text, String scripting) {
    setId(name);
    name_ = name;
    value_ = text;
    scripting_ = scripting;
    editable_ = true;
    setRendererType("StringInputRenderer");
    FacesUtil.createValueBinding(getFacesContext(), this, "text", text);
  }

  public String getFamily() {
    return COMPONENT_FAMILY;
  }

  final public String getText() {
    return value_;
  }

  final public UIStringInput setText(String s) {
    value_ = s;
    return this;
  }

  final public String getTitle() {
    return title_;
  }

  final public UIStringInput setTitle(String title) {
    title_ = title;
    return this;
  }

  public int getType() {
    return type;
  }

  public UIStringInput setType(int type) {
    this.type = type;
    return this;
  }

  public void reset() {
    value_ = "";
  }

  public UIStringInput addValidator(Validator validator) {
    addComponentValidator(validator);
    return this;
  }

  public UIStringInput addValidator(Class clazz) {
    addComponentValidator(clazz);
    return this;
  }

  final public void processValidators(FacesContext context) {
    processValidators(context, value_);
  }

  public void decodeFromMultipartFields(FacesContext context,
      UIComponent component, List items) {
    Iterator iter = items.iterator();
    while (iter.hasNext()) {
      FileItem item = (FileItem) iter.next();
      if (item.isFormField()) {
        String name = item.getFieldName();
        if (name.equals(getName())) {
          String value = item.getString();
          if (value == null)
            return;
          // Update managed bean (model object)
          if (!FacesUtil.updateBoundValueBinding(context, component, "text",
              value)) {
            setText(value);
          }
        }
      }
    }
  }
}