/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portal.faces.component;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.faces.component.UIComponent;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.portal.model.Container;
import org.exoplatform.services.portal.skin.SkinConfigService;
import org.exoplatform.services.portal.skin.model.Decorator;
/**
 * Sat, Jan 03, 2004 @ 11:16 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIContainerForm.java,v 1.11 2004/11/03 01:19:45 tuan08 Exp $
 */
public class UIContainerForm extends UISimpleForm {
  final static String VIEW_PERMISSION = "viewPermission", EDIT_PERMISSION = "editPermission" ,
                      TITLE = "title", ICON = "icon", STATE = "state", DECORATOR = "decorator", 
                      RENDERER = "renderer", WIDTH = "width", HEIGHT = "height", ID = "id";
  
  private UIContainer uiEditingContainer_ ; 
  UIComponent returnComponent_ ;
  private UIPortalComponent uiParent_ ; 
  
  public UIContainerForm(SkinConfigService skinService)  {
    super("containerForm", "post", null) ;
    setId("UIContainerForm");
    setClazz("UIExoForm") ;
    setRendererType("SimpleFormVelocityRenderer");
    List<SelectItem> rendererOptions = new ArrayList<SelectItem>() ;
    Iterator i = skinService.getContainerDecorators().iterator() ; 
    while(i.hasNext()) {
      Decorator decorator = (Decorator) i.next() ;
      String rendererType = decorator.getRendererType() ;
      rendererOptions.add(new SelectItem(rendererType, rendererType)) ;
    }
    add(new UIStringInput(ID, "").setEditable(false)) ; 
    add(new UIStringInput(TITLE, "")) ;
    add(new UISelectBox(DECORATOR, "", new ArrayList())) ;
    UISelectBox rendererInput = new UISelectBox("renderer","" , rendererOptions) ;
    rendererInput.setUpdateOnChangeAction("updateStyles");
    add(rendererInput) ;
    add(new UIStringInput(WIDTH , "")) ;
    add(new UIStringInput(HEIGHT, "")) ;
    addActionListener(UpdateRendererActionListener.class, "updateStyles") ;
    addActionListener(SaveActionListener.class, SAVE_ACTION) ;
    addActionListener(CancelActionListener.class, CANCEL_ACTION) ;
  }
  
  public void  setEditingContainer(UIComponent returnComponent, UIContainer uiContainer) {
    returnComponent_ = returnComponent ;
    uiEditingContainer_ = uiContainer ;
    SkinConfigService skinService =  
      (SkinConfigService)PortalContainer.getComponent(SkinConfigService.class) ;
    Container model = uiContainer.getContainerModel() ;
    String currentRenderer = model.getRenderer() ;
    if(currentRenderer == null)  currentRenderer = uiContainer.getDefaultRendererName() ;

    String currentDecorator = uiContainer.getDecorator() ;
    if(currentDecorator == null || currentDecorator.length() == 0) {
      currentDecorator = "default" ;
    }

    UISelectBox uiSelectDecorator = getUISelectBox(DECORATOR) ;
    Decorator decorator = skinService.getContainerDecorator(currentRenderer) ; 
    Util.updateSelectDecorator(uiSelectDecorator, decorator, currentDecorator) ;
    
    getUIStringInput(ID).setValue(uiContainer.getId());
    getUISelectBox(RENDERER).setValue(currentRenderer);
    getUISelectBox(DECORATOR).setValue(currentDecorator) ;
    getUIStringInput(WIDTH).setValue(model.getWidth()) ;
    getUIStringInput(HEIGHT).setValue(model.getHeight()) ;
  }
  
  public void  addNewContainer(UIComponent returnComponent, UIPortalComponent uiParent) {
    returnComponent_ = returnComponent ;
    uiEditingContainer_ = null ;
    uiParent_ = uiParent ;
    SkinConfigService skinService =  
      (SkinConfigService)PortalContainer.getComponent(SkinConfigService.class) ;
    UISelectBox uiSelectDecorator = getUISelectBox(DECORATOR) ;
    Decorator decorator = skinService.getContainerDecorator("default") ; 
    Util.updateSelectDecorator(uiSelectDecorator, decorator, null) ;
    getUIStringInput(ID).setValue("");
    getUISelectBox(RENDERER).setValue("default");
    getUIStringInput(WIDTH).setValue("") ;
    getUIStringInput(HEIGHT).setValue("") ;
  }
  
  static public class UpdateRendererActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIContainerForm uiForm = (UIContainerForm) event.getSource() ;
      String selectRenderer = uiForm.getUISelectBox(RENDERER).getValue() ;
      SkinConfigService skinService =  
        (SkinConfigService)PortalContainer.getComponent(SkinConfigService.class) ;
      UISelectBox uiSelectDecorator = uiForm.getUISelectBox(DECORATOR) ;
      Decorator decorator = skinService.getContainerDecorator(selectRenderer) ; 
      Util.updateSelectDecorator(uiSelectDecorator, decorator, null) ;
    }
  }
  
  static public class SaveActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
      UIContainerForm uiForm = (UIContainerForm) event.getSource() ;
			UIPortal uiPortal = (UIPortal) uiForm.getAncestorOfType(UIPortal.class) ;
		  if(uiForm.uiEditingContainer_ == null) {	
        List children = uiForm.uiParent_.getChildren() ;
        uiForm.uiEditingContainer_ = new UIContainer(new Container()) ;
        children.add(uiForm.uiEditingContainer_) ;
      }
		  Container model = uiForm.uiEditingContainer_.getEditableContainerModel() ;
      model.setDecorator(uiForm.getUISelectBox(DECORATOR).getValue()) ;
      model.setRenderer(uiForm.getUISelectBox(RENDERER).getValue()) ;
      model.setWidth(uiForm.getUIStringInput(WIDTH).getValue()) ;
      model.setHeight(uiForm.getUIStringInput(HEIGHT).getValue()) ;
      model.setId(uiForm.getUIStringInput(ID).getValue()) ;
      model.setTitle(uiForm.getUIStringInput(TITLE).getValue()) ;      
      model.setId(uiForm.uiEditingContainer_.getId()) ;
      uiForm.uiEditingContainer_.updateChange() ;
      uiForm.uiEditingContainer_.setDisplayTitle(model.getTitle()) ;
      uiForm.uiEditingContainer_.setId(uiForm.getUIStringInput(ID).getValue()) ;
      uiForm.uiEditingContainer_.setComponentModified(true) ;
      uiPortal.setBodyComponent(uiForm.returnComponent_) ;
		}
	}
  
  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIContainerForm uiForm = (UIContainerForm) event.getSource() ;
      UIPortal uiPortal = (UIPortal) uiForm.getAncestorOfType(UIPortal.class)  ;
      uiPortal.setBodyComponent(uiForm.returnComponent_) ;
    }
  }
}