package org.exoplatform.portal.faces.component;

import java.util.List;
import java.util.ResourceBundle;

import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.portal.Information;
import org.exoplatform.portal.PortalConstants;
import org.exoplatform.portal.faces.listener.portal.NodeActionListener;
import org.exoplatform.services.portal.model.Node;
/**
 * @version $Revision: 1.11 $ $Date: 2004/09/27 18:26:58 $
 * @author Fahrid Djebbari
 */
public class UINavigationManager extends UINavigation { 
  static String SElECT_NODE = "selectNode", COPY = "copy", PASTE = "paste"  ;
  static Parameter[] MOVE_NODE_UP_PARAMS = {new Parameter(ACTION, PortalConstants.MOVE_UP_NODE_ACTION) };
  static Parameter[] MOVE_NODE_DOWN_PARAMS = {new Parameter(ACTION, PortalConstants.MOVE_DOWN_NODE_ACTION) };
  static Parameter[] EDIT_NODE_PARAMS = {new Parameter(ACTION, PortalConstants.EDIT_NODE_ACTION) };
  static Parameter[] DELETE_NODE_PARAMS = {new Parameter(ACTION, PortalConstants.DELETE_NODE_ACTION) };
  static Parameter[] ADD_NODE_PARAMS = {new Parameter(ACTION, PortalConstants.ADD_NODE_ACTION) };
  static Parameter[] SELECT_NODE_PARAMS = {new Parameter(ACTION, SElECT_NODE) };
  static Parameter[] COPY_NODE_PARAMS = {new Parameter(ACTION, COPY) };
  static Parameter[] PASTE_NODE_PARAMS = {new Parameter(ACTION, PASTE) };
  
  private Node copyNode =  null; 
  
	public UINavigationManager() throws Exception {
	  super() ;
    setId("UINavigationManager") ;
    setRendererType("VelocityRenderer") ;
    
    addActionListener(NavigationActionListener.class, PortalConstants.EDIT_NODE_ACTION) ;
    addActionListener(NavigationActionListener.class, PortalConstants.ADD_NODE_ACTION) ;
    addActionListener(NavigationActionListener.class, PortalConstants.DELETE_NODE_ACTION) ;
    addActionListener(NavigationActionListener.class, PortalConstants.MOVE_UP_NODE_ACTION) ;
    addActionListener(NavigationActionListener.class, PortalConstants.MOVE_DOWN_NODE_ACTION) ;
    addActionListener(SelectNodeActionListener.class, SElECT_NODE) ;
    addActionListener(CopyNodeActionListener.class, COPY) ;
    addActionListener(PasteNodeActionListener.class, PASTE) ;
	}
  
  public Parameter[] getMoveUpParams() { return MOVE_NODE_UP_PARAMS ; }
  public Parameter[] getMoveDownParams() { return MOVE_NODE_DOWN_PARAMS ; }
  public Parameter[] getEditNodeParams() { return EDIT_NODE_PARAMS ; }
  public Parameter[] getDeleteNodeParams() { return DELETE_NODE_PARAMS ; }
  public Parameter[] getAddNodeParams() { return ADD_NODE_PARAMS ; }
  public Parameter[] getSelectNodeParams() { return SELECT_NODE_PARAMS ; }
  public Parameter[] getCopyParams() { return COPY_NODE_PARAMS ; }
  public Parameter[] getPasteParams() { return PASTE_NODE_PARAMS ; }
  
  static public class SelectNodeActionListener extends NodeActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UINavigationManager uiNav = (UINavigationManager) event.getSource() ;
      UIPortal uiPortal = (UIPortal) uiNav.getAncestorOfType(UIPortal.class) ;
      Node root = uiPortal.getRootNode() ;
      String uri = event.getParameter(OBJECTID) ;
      Node node = root.findNode(uri) ;
      node.setExpanded(!node.isExpanded()) ;
    }
  }
  
  static public class CopyNodeActionListener extends NodeActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UINavigationManager uiNav = (UINavigationManager) event.getSource() ;
      UIPortal uiPortal = (UIPortal) uiNav.getAncestorOfType(UIPortal.class) ;
      Node root =  uiPortal.getRootNode() ;
      uiNav.copyNode = root.findNode(event.getParameter(OBJECTID)).cloneNode() ;
    }
  }
  
  static public class PasteNodeActionListener extends NodeActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UINavigationManager uiNav = (UINavigationManager) event.getSource() ;
      Information ip = findInformationProvider(uiNav);
      ResourceBundle res = getApplicationResourceBundle() ;
      UIPortal uiPortal = (UIPortal) uiNav.getAncestorOfType(UIPortal.class) ;
      if(uiNav.copyNode == null) {
        ip.addMessage(new ExoFacesMessage(res.getString("UINavigationManager.msg.no-copied-node")));
        return ;
      }
      Node root =  uiPortal.getRootNode() ;
      Node target = root.findNode(event.getParameter(OBJECTID));
      if(target.isShare()) {
        ip.addMessage(new ExoFacesMessage(res.getString("UINavigationManager.msg.no-write-permission")));
        return ;
      }
      Node src = uiNav.copyNode.cloneNode() ;
      if(target.hasChild(src.getName())) {
        ip.addMessage(new ExoFacesMessage(res.getString("UINavigationManager.msg.duplicate-node-name")));
        return ;
      }
      resetURI(target, src) ;
      target.addChild(src) ;
    }
    
    private void resetURI(Node parent, Node child) {
      String uri = parent.getUri() ;
      if(uri.equals("/")) uri += child.getName()  ;
      else uri += "/" + child.getName()  ;
      child.setUri(uri) ;
      child.setParent(parent) ;
      child.setLevel(parent.getLevel() + 1) ;
      parent = child ;
      for(int i = 0 ; i < parent.getChildrenSize(); i++ ) {
        child = parent.getChild(i) ;
        resetURI(parent, child) ;
      }
    }
  }
  
  static public class NavigationActionListener extends NodeActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UINavigationManager uiNav = (UINavigationManager) event.getSource() ;
      UIPortal uiPortal = (UIPortal) uiNav.getAncestorOfType(UIPortal.class) ;
      Information iprovider = findInformationProvider(uiNav);
      ResourceBundle res = getApplicationResourceBundle() ;
      String portalAction = event.getAction() ;
      if (PortalConstants.EDIT_NODE_ACTION.equals(portalAction)) {
        UIPageNodeForm uiPageNodeForm = 
          (UIPageNodeForm)PortalComponentCache.findPortalComponent(UIPageNodeForm.class) ;
        Node root = uiPortal.getRootNode() ;
        Node node = root.findNode(event.getParameter(OBJECTID)) ;
        uiPageNodeForm.setEditingNavigationNode(null, node);
        uiPortal.setBodyComponent(uiPageNodeForm);
      } else if (PortalConstants.ADD_NODE_ACTION.equals(portalAction)) {
        UIPageNodeForm uiPageNodeForm = 
          (UIPageNodeForm)PortalComponentCache.findPortalComponent(UIPageNodeForm.class) ;
        Node root = uiPortal.getRootNode() ;
        Node node = root.findNode(event.getParameter(OBJECTID)) ;
        uiPageNodeForm.setEditingNavigationNode(node, null);
        uiPortal.setBodyComponent(uiPageNodeForm);          
      } else if (PortalConstants.DELETE_NODE_ACTION.equals(portalAction)) {
        Node root = uiPortal.getRootNode() ;
        Node node = root.findNode(event.getParameter(OBJECTID)) ;
        if(node.isShare()) {
          iprovider.addMessage(new ExoFacesMessage(res.getString("UINavigationManager.msg.delete-failed"))) ;
        }else{
          Node parent = node.getParent() ;
          if(parent != null) parent.removeChild(node.getUri()) ;
          uiPortal.setSelectedNode(parent) ;
        }        
      } else if (PortalConstants.MOVE_UP_NODE_ACTION.equals(portalAction)) {
        Node root = uiPortal.getRootNode() ;
        Node node = root.findNode(event.getParameter(OBJECTID)) ;
        if(node != null) moveUp(node) ;
      } else if (PortalConstants.MOVE_DOWN_NODE_ACTION.equals(portalAction)) {
        Node root = uiPortal.getRootNode() ;
        Node node = root.findNode(event.getParameter(OBJECTID)) ;
        if(node != null) moveDown(node) ;
      } 
    }
    
    public void moveUp(Node node) {
      Node parent = node.getParent();
      if(parent == null ) return ;
      List children = parent.getChildren() ;
      for (int i = 0; i < children.size(); i++) {
        Node child =  (Node)children.get(i);
        if (child == node) {
          if(i != 0) {
            Object tmp = children.remove(i);
            children.add(i - 1, tmp);
          }
          return;
        }
      }
    }

    public void moveDown(Node node) {
      Node parent = node.getParent();
      if(parent == null ) return ;
      List children = parent.getChildren() ;
      for (int i = 0; i < children.size(); i++) {
        Node child = (Node) children.get(i);
        if (child == node) {
          if(i != children.size() - 1) {
            Object tmp = children.remove(i);
            children.add(i + 1, tmp);
          }
          return;
        }
      }
    }
  }
}