/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portal.faces.component.logic;

import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.portal.faces.component.UIBody;
import org.exoplatform.portal.faces.component.UIPortalComponent;
import org.exoplatform.portal.faces.component.UIPortlet;
import org.exoplatform.services.portal.skin.SkinConfigService;
import org.exoplatform.services.portal.skin.model.Style;
import org.exoplatform.services.portletcontainer.pci.ExoWindowID;
/**
 * @author Tuan Nguyen (tuan08@users.sourceforge.net)
 * @since Nov 17, 2004
 * @version $Id$
 */
public class PortalSkin  {
  private SkinConfigService service_ ;
  private Map<String,String> cssURLMap_ ;
  
  public PortalSkin() {
    service_ = 
      (SkinConfigService) PortalContainer.getComponent(SkinConfigService.class) ;
    cssURLMap_ = new LinkedHashMap<String,String>(15) ;
  }
  
  public String  getCSS(UIPortalComponent root) {
    addBasicComponentStyle(root) ;
    List descendants = root.findDescendantsOfType(UIPortalComponent.class) ;
    for(int i = 0; i < descendants.size(); i++) {
      UIPortalComponent descendant = (UIPortalComponent) descendants.get(i) ;
      if(descendant instanceof UIBody) continue ;
      addBasicComponentStyle(descendant) ;
      if(descendant instanceof UIPortlet) {
        addPortletComponentStyle((UIPortlet) descendant) ;
      }
    }
    StringBuffer sbuf = new StringBuffer() ;
    Iterator styleIterator = cssURLMap_.values().iterator();
    while (styleIterator.hasNext()) {
      String cssLink = (String) styleIterator.next() ;
      sbuf.append(cssLink);
    }
    return sbuf.toString() ;
  }
 
  private void addPortletComponentStyle(UIPortlet comp) {
    String portletStyle = comp.getPortletModel().getPortletStyle() ;
    //if (portletStyle == null  || portletStyle.length() == 0 || "default".equals(portletStyle)) return ;
    ExoWindowID windowId = comp.getWindowId() ;
    String portletId = windowId.getPortletApplicationName() + "/" +windowId.getPortletName() ;
    //try to lookup or use the default styles in the same portlet id configuration.
    Style styleConfig = service_.getPortletStyle(portletId, portletStyle) ;
    //try the styles configuration in the default  portlet skin configuration
    if(styleConfig == null) {
      styleConfig = service_.getPortletStyle("default", portletStyle) ;
    }
    if (styleConfig == null) {
      String cssLink = "expect an style url here for the  portlet : " +
      portletId + " style " + portletStyle  + '\n';
      String key =  portletId + "/" + portletStyle ;
      cssURLMap_.put(key , cssLink) ;
    } else {
      String url = styleConfig.getUrl() ;
      String cssLink = "<link rel='stylesheet' type='text/css' href='" + url + "'/>\n" ;
      cssURLMap_.put(url, cssLink) ;
    }
  }
  
  private void addBasicComponentStyle(UIPortalComponent comp) {
    String decorator = comp.getDecorator() ;
    if (decorator != null  && decorator.length() > 0 && !"default".equals(decorator)) {
      String rendererRef = comp.getComponentModel().getRenderer() ;
      if(rendererRef == null || rendererRef.length() == 0) {
        rendererRef = comp.getDefaultRendererName() ;
      }
      Style styleConfig = comp.getDecoratorStyle(service_, rendererRef, decorator) ;
      if (styleConfig == null) {
        String cssLink = "expect an style url here, component type:" + comp.getClass().getName() +
                         ", renderer: " + rendererRef + ", decorator: " + decorator  + '\n';
        String key = comp.getIdPrefix() + "/" + rendererRef + "/" + decorator ;
        cssURLMap_.put(key , cssLink) ;
      } else {
        String url = styleConfig.getUrl() ;
        String cssLink = "<link rel='stylesheet' type='text/css' href='" + url + "'/>\n" ;
        cssURLMap_.put(url, cssLink) ;
      }
    }
  }
}