/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portal.faces.renderer.html.portlet;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.ResourceBundle;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.portlet.WindowState;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.exoplatform.Constants;
import org.exoplatform.commons.utils.ExceptionUtil;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.container.SessionContainer;
import org.exoplatform.faces.core.component.UIExoComponent;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.portal.PortalConstants;
import org.exoplatform.portal.faces.component.UIPortalComponent;
import org.exoplatform.portal.faces.component.UIPortlet;
import org.exoplatform.portal.faces.renderer.BaseRenderer;
import org.exoplatform.portal.session.RequestInfo;
import org.exoplatform.services.log.LogUtil;
import org.exoplatform.services.organization.UserProfile;
import org.exoplatform.services.portletcontainer.PortletContainerService;
import org.exoplatform.services.portletcontainer.pci.ExoWindowID;
import org.exoplatform.services.portletcontainer.pci.RenderInput;
import org.exoplatform.services.portletcontainer.pci.RenderOutput;
/**
 * Fri, May 30, 2003 @
 * @author : Mestrallet Benjamin
 * @email:   benjmestrallet@users.sourceforge.net
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: PortletRenderer.java,v 1.19 2004/11/01 21:06:50 tuan08 Exp $
 */
public class PortletRenderer extends BaseRenderer {
  private PortletContainerService portletContainer_ ;
  
  protected void decodeComponentAction(FacesContext context, UIComponent uiComponent) {
    UIPortlet basicComponent = (UIPortlet) uiComponent ;
    Map paramMap = context.getExternalContext().getRequestParameterMap() ;
    basicComponent.broadcast(new ExoActionEvent(uiComponent, "portletAction", paramMap)) ;
  }
  
  public void encodeChildren( FacesContext context, UIComponent component ) throws IOException {
    ResourceBundle res = getApplicationResourceBundle(context.getExternalContext()) ;
    ResponseWriter w = context.getResponseWriter();
    UIPortlet uiPortlet = (UIPortlet) component ;
    try {
      VelocityContext vcontext = new VelocityContext() ;
      vcontext.put(UIExoComponent.UICOMPONENT, uiPortlet) ;
      vcontext.put("context", context) ;
      vcontext.put("res", res) ;
      vcontext.put("renderer", this) ;
      vcontext.put("componentId", uiPortlet.getId()) ;
      vcontext.put("model", uiPortlet.getComponentModel()) ;
      if(uiPortlet.getMode() == UIPortalComponent.COMPONENT_EDIT_MODE) {
        vcontext.put("mode", "edit") ;
        vcontext.put("editParams", PortalConstants.editParams);
        vcontext.put("deleteParams", PortalConstants.deleteParams);
        if(uiPortlet.getFloat() == UIPortalComponent.FLOAT_BOTTOM) { 
          vcontext.put("float", "bottom") ;
          vcontext.put("moveUpParams", PortalConstants.moveUpParams);
          vcontext.put("moveDownParams", PortalConstants.moveDownParams);
        } else {
          vcontext.put("float", "right") ;
          vcontext.put("moveLeftParams", PortalConstants.moveUpParams);
          vcontext.put("moveRightParams",  PortalConstants.moveDownParams);
        }
      } else {
        vcontext.put("mode", "view") ;
      }
      addPortletData(vcontext, context, uiPortlet) ;
      Template template = getTemplate(component, context) ;
      template.merge(vcontext, w) ;
    } catch (Exception ex) {
      LogUtil.getLog(getClass()).error("Render  Error: ", ex) ;
    }
  }
  
  public void addPortletData(VelocityContext vcontext, FacesContext fcontext, UIPortlet uiPortlet) {
    if(uiPortlet.hasError())  return ;
    PortletContainerService portletContainer = getPortletContainerService() ;
    HttpServletRequest request = (HttpServletRequest) fcontext.getExternalContext().getRequest() ;
    HttpServletResponse response = (HttpServletResponse) fcontext.getExternalContext().getResponse();
    SessionContainer scontainer = SessionContainer.getInstance() ;
    RequestInfo rinfo = (RequestInfo) scontainer.getComponentInstanceOfType(RequestInfo.class);
    UserProfile uprofile = 
      (UserProfile) scontainer.getComponentInstanceOfType(UserProfile.class) ;
    
    Map renderParams = uiPortlet.getRenderParameters() ;
    if (renderParams == null) {
      renderParams = copyRequestParameterMap(request.getParameterMap()) ;
      uiPortlet.setRenderParameters(renderParams);
    }
    RenderInput input = new RenderInput();
    String baseUrl = null ;
    String configurationSource = uiPortlet.getWindowId().getConfigurationSource() ;
    if(configurationSource == ExoWindowID.DEFAULT_PORTAL_CONFIG ||
       configurationSource == ExoWindowID.MOBILE_PORTAL_CONFIG ) {
      baseUrl  = new StringBuilder(rinfo.getOwnerURI()).
                 append("?" + Constants.COMPONENT_PARAMETER).append("=").
                 append(uiPortlet.getWindowId().getUniqueID()).toString() ;
    } else {
      baseUrl = new StringBuffer(rinfo.getNodeURI()).
                append("?" + Constants.COMPONENT_PARAMETER).append("=").
                append(uiPortlet.getWindowId().getUniqueID()).toString() ;
    }
    input.setBaseURL(baseUrl);
    input.setWindowID(uiPortlet.getWindowId());
    input.setUserAttributes(uprofile.getUserInfoMap());
    input.setPortletMode(uiPortlet.getPortletMode());
    input.setWindowState(uiPortlet.getWindowState());
    input.setMarkup("text/html");
    input.setRenderParameters(renderParams);
    input.setTitle(uiPortlet.getDisplayTitle());
    input.setUpdateCache(uiPortlet.getUpdateCache());
    RenderOutput output = null;
    String portletContent = "EXO-ERROR: Portlet container throw an exception\n" ;
    String  portletTitle = null ;
    try {
      output = portletContainer.render(request, response, input);
      uiPortlet.setUpdateCache(false) ;
      if(output.getContent() == null) {
        portletContent = uiPortlet.getId() + " has error";
      } else {
        portletContent = new String(output.getContent()) ;
      }
    } catch (Throwable ex) {
      ex = ExceptionUtil.getRootCause(ex) ;
      portletContent +=  ExceptionUtil.getStackTrace(ex, 100) ;
      LogUtil.getLog(getClass()).error("Render Portlet Error: ", ex) ;
    }
    if (portletContent == null) portletContent = "" ;

    if(output != null ) portletTitle = output.getTitle() ;
    if(portletTitle == null ) portletTitle = "Portlet" ;
    vcontext.put("portletContent", portletContent) ;
    vcontext.put("portletTitle", portletTitle) ;
    vcontext.put("baseUrl", baseUrl) ;
  }
 
  
  public String getShowMode(UIPortlet uiPortlet, String mode) {
    String currentMode = uiPortlet.getPortletMode().toString() ;
    if(currentMode.equals(mode)) return UIPortlet.PORTLET_VIEW_MODE ;
    return mode ;
  }
  
  public String getShowState(UIPortlet portlet, String state) {
    String currentState = portlet.getWindowState().toString()  ;
    if(currentState.equals(state)) state = WindowState.NORMAL.toString()  ;
    return state ;
  }

  private PortletContainerService getPortletContainerService() {
    if ( portletContainer_ == null) {
      portletContainer_ = 
        (PortletContainerService) PortalContainer.getInstance().
                                  getComponentInstanceOfType(PortletContainerService.class);
    } 
    return portletContainer_ ;
  }
  
  private Map copyRequestParameterMap(Map map) {
    Map temp = new HashMap(10) ;
    Iterator keys = map.keySet().iterator() ;
    while (keys.hasNext()) {
      String key = (String) keys.next() ;
      temp.put(key, map.get(key)) ;
    }
    return temp ;
  }
}
