/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.indexing.component;

import java.util.Collection;
import java.util.ResourceBundle;

import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UIPortlet;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.CheckRoleInterceptor;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.services.indexing.IndexerPlugin;
import org.exoplatform.services.indexing.IndexingService;
/**
 * Sat, Jan 03, 2004 @ 11:16 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIListIndexer.java,v 1.2 2004/09/16 20:17:02 tuan08 Exp $
 */
public class UIListIndexer extends UIExoCommand {
  final static public String REINDEX_ACTION = "reindex" ;
  final static public String REMOVE_INDEX_ACTION = "removeIndex" ;
  final static public String MORE_ACTION = "more" ;
	static Parameter[] REMOVE_INDEX_PARAMS =  { new Parameter(ACTION , REMOVE_INDEX_ACTION) } ;
  static Parameter[] REINDEX_PARAMS =  { new Parameter(ACTION , REINDEX_ACTION) } ;
  static Parameter[] MORE_PARAMS =  { new Parameter(ACTION , MORE_ACTION) } ;
	
	private IndexingService iservice_ ; 
	
	public UIListIndexer(IndexingService iservice) throws Exception {
		setId("UIListIndexer") ;	
    setRendererType("VelocityRenderer");
    iservice_ = iservice ;
		addActionListener(ReindexActionListener.class, REINDEX_ACTION ) ;
    addActionListener(RemoveIndexActionListener.class, REMOVE_INDEX_ACTION ) ;
		addActionListener(MoreActionListener.class, MORE_ACTION) ;
	}
  public Collection getDataHandler() throws Exception { return iservice_.getIndexerPlugins() ; }
  public Parameter[] getReIndexParams() { return REINDEX_PARAMS ; }
  public Parameter[] getRemoveIndexParams() { return REMOVE_INDEX_PARAMS ; }
  public Parameter[] getMoreParams() { return MORE_PARAMS ; }  
  
	static public class MoreActionListener extends ExoActionListener  {
    public MoreActionListener() {
      addInterceptor(new CheckRoleInterceptor("admin")) ;  
    }
    
    public void execute(ExoActionEvent event) throws Exception {
      UIListIndexer uiList= (UIListIndexer) event.getComponent() ;
      String objectId = event.getParameter(OBJECTID) ;
      UIPortlet uiPortlet = (UIPortlet)  uiList.getParent() ;
      if(uiPortlet.findComponentById(objectId) == null) {
        Information iprovider = findInformationProvider(uiList) ;
        ResourceBundle res = getApplicationResourceBundle() ;
        iprovider.addMessage(new ExoFacesMessage(res.getString("UIListIndexer.msg.not-support-extension"))) ;
      } else {
        uiPortlet.setRenderedComponent(objectId) ;
      }
    }
  }
  
	static public class ReindexActionListener extends ExoActionListener  {
    public ReindexActionListener() {
      addInterceptor(new CheckRoleInterceptor("admin")) ;  
    }
    
		public void execute(ExoActionEvent event) throws Exception {
      UIListIndexer uiList = (UIListIndexer) event.getComponent() ;
      String objectId = event.getParameter(OBJECTID) ;
      uiList.iservice_.reindex(objectId) ;
		}
	}
  
  static public class RemoveIndexActionListener extends ExoActionListener  {
    public RemoveIndexActionListener() {
      addInterceptor(new CheckRoleInterceptor("admin")) ;  
    }
    
    public void execute(ExoActionEvent event) throws Exception {
      UIListIndexer uiList = (UIListIndexer) event.getComponent() ;
      String objectId = event.getParameter(OBJECTID) ;
      IndexerPlugin plugin = uiList.iservice_.getIndexerPlugin(objectId) ;
      plugin.removeIndex() ;
    }
  }
}