/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.portal.content.impl;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.servlet.ServletContext;

import org.apache.commons.lang.StringUtils;
import org.exoplatform.container.component.ComponentPlugin;
import org.exoplatform.services.cache.CacheService;
import org.exoplatform.services.cache.FIFOExoCache;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.database.ObjectQuery;
import org.exoplatform.services.portal.content.ContentPlugin;
import org.exoplatform.services.portal.content.DocumentContent;
import org.exoplatform.services.portal.content.PortalContentService;
import org.exoplatform.services.resources.LocaleConfig;
import org.exoplatform.services.resources.LocaleConfigService;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.picocontainer.Startable;


/**
 * Created by The eXo Platform SARL
 * Author : Pham Xuan Hoa
 *          phamvuxuanhoa@yahoo.com
 * Feb 23, 2006
 */
public class PortalContentServiceImpl implements PortalContentService ,Startable, ComponentPlugin{
	
	final private static String findAllDocuments = 
		"from d in class org.exoplatform.services.portal.content.impl.DocumentContentImpl";
	
	final private static String findDocumentById = 
		"from d in class org.exoplatform.services.portal.content.impl.DocumentContentImpl" +
		" where d.id = ? ";
	
	final private static String findDocumentsByOwner = 
		"from d in class org.exoplatform.services.portal.content.impl.DocumentContentImpl" +
		" where d.owner = ? ";
	
	final private static String ENCODING = "UTF-8" ;
	private HibernateService hservice_ ;  
	private ServletContext sContext_ ;	
	private LocaleConfigService localeConfigService_ ;
	private FIFOExoCache contentCache_ ;
	
	public PortalContentServiceImpl(HibernateService hservice,ServletContext servletContext,
			CacheService cacheService,LocaleConfigService localeConfigService) throws Exception {    
		hservice_ = hservice ;                
		sContext_ = servletContext ;		
		localeConfigService_ = localeConfigService ;
		contentCache_ = new FIFOExoCache(20);
	}
	
	public void addContentPlugin(ComponentPlugin plugin) throws Exception {	  
		ContentPlugin contentPlugin = (ContentPlugin)plugin ;	 
		initDatabase(contentPlugin) ;
	}  
	
	public DocumentContent createDocumentContentInstance() {
		return new DocumentContentImpl() ;
	}
	
	public DocumentContent getDocumentById(String id) throws Exception {
		DocumentContent document = (DocumentContent)contentCache_.get(id) ;
		if(document!=null) return document ;		
		Session session = hservice_.openSession() ;
		document = (DocumentContent)hservice_.findOne(session,findDocumentById,id) ;
		contentCache_.put(id,document) ;				    
		return document ;
	}
	
	public List<DocumentContent> getDocumentsByGroupId(String groupId) throws Exception {	
		List<DocumentContent> documents = new ArrayList<DocumentContent>() ;
		for(Object obj:contentCache_.getCachedObjects()) {
			DocumentContent tmp = (DocumentContent)obj ;
			String tempId = StringUtils.substringBeforeLast(tmp.getId(),"_") ;			
			if(tempId.equals(groupId)) documents.add(tmp) ; 
		}
		if(!documents.isEmpty()) return documents ;
		ObjectQuery query = new ObjectQuery(DocumentContent.class) ;
		query.addLIKE("id",groupId+"%") ;					
		Session session = hservice_.openSession() ;		
		documents = session.createQuery(query.getHibernateQuery()).list() ; ;			
		return documents;
	}
	
	public void save(DocumentContent document) throws Exception {    
		Session session = hservice_.openSession() ;
		Transaction transaction = session.beginTransaction() ;
		hservice_.save(document) ;
		transaction.commit() ;    
	}
	
	public List<DocumentContent> getDocumentsByOwner(String owner) throws Exception {
		Session session = hservice_.openSession() ;
		List<DocumentContent> documents = 
			session.createQuery(findDocumentsByOwner).list() ; ;      
			return documents;
	}
	
	public List<DocumentContent> getAllDocuments() throws Exception {
		Session session = hservice_.openSession() ;
		List<DocumentContent> documents = session.createQuery(findAllDocuments).list() ;
		return documents;
	}
	
	public void update(DocumentContent document) throws Exception { 
		Session session = hservice_.openSession() ;
		Transaction transaction = session.beginTransaction() ;
		session.update(document) ;    
		transaction.commit() ;
	}
	
	public void remove(DocumentContent document) throws Exception {
		Session session = hservice_.openSession() ;
		Transaction transaction = session.beginTransaction() ;
		session.delete(document) ;
		transaction.commit();   
	}
	
	public void initDatabase(ContentPlugin plugin) throws Exception{
		String  content = null ;   
		String owner = plugin.getContentOwner() ;		
		Collection<LocaleConfig> configs = localeConfigService_.getLocalConfigs() ;		
		List<String> contentIds = plugin.getContentIds() ;
		for(String id:contentIds) {						
			if(id.startsWith("war:")||id.startsWith("file://")) {
				if (id.endsWith(".html")) {
					try {      
						content = null ;
						content = getExternalContent(id,ENCODING) ;            
					} catch (Exception ex) { }
					if(content !=null) {      
						storeContent(id,content,owner) ;
					}
				} 
				else{
					String tmpId = id + ".html" ;
					try {
						content = null ;
						content = getExternalContent(tmpId,ENCODING);
					}catch (Exception e) { }
					if(content!=null) {
						String realId = id + "_en.html" ;
						storeContent(realId,content,owner) ;						
					}
					for (LocaleConfig localeConfig:configs) {						
						String tmpUri = id + "_"+localeConfig.getLanguage()+".html";						
						try {
							content = null ;
							content = getExternalContent(tmpUri,ENCODING);
						} catch (Exception ex) { }
						if(content !=null) {  							
							storeContent(tmpUri,content,owner) ;
						}
					}					
				} 
			}						      
		}
	}
	
	private void  storeContent(String id,String content,String owner) throws Exception{
		DocumentContent documentContent = createDocumentContentInstance() ;	
		documentContent.setId(id) ;
		documentContent.setContent(content) ;
		documentContent.setOwner(owner) ;
		save(documentContent) ;		
	}
	
	public String getExternalContent(String uri, String encoding) throws Exception {
		String content = null ;
		if (uri.startsWith("war:")) {
			String path = uri.substring(4, uri.length());               
			InputStream is = sContext_.getResourceAsStream(path);  
			if (is == null) throw new IOException();
			byte[] buf = new byte[is.available()];
			is.read(buf);
			content = new String(buf, encoding);
		} else if (uri.startsWith("file://")) {
			String path = uri.substring(7, uri.length());   			
			FileInputStream is = new FileInputStream(path);
			if (is == null)
				throw new IOException();
			byte[] buf = new byte[is.available()];
			is.read(buf);
			content = new String(buf, encoding);
		}
		return content;
	}
	
	public void start() { }	
	public void stop() { }
	
	public String getName() {	return null; }	
	public void setName(String s) {}
	
	public String getDescription() { return null;	}	
	public void setDescription(String s) { 	}
	
}
