/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.portal.log.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.GregorianCalendar;
import java.util.List;

import org.exoplatform.commons.utils.PageList;
import org.exoplatform.container.component.BaseComponentPlugin;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.services.cache.CacheListener;
import org.exoplatform.services.cache.CacheService;
import org.exoplatform.services.cache.ExoCache;
import org.exoplatform.services.cache.FIFOExoCache;
import org.exoplatform.services.database.DBObjectPageList;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.database.ObjectQuery;
import org.exoplatform.services.portal.log.UserActivitiesLogData;
import org.exoplatform.services.portal.log.UserLogQuery;
import org.hibernate.Session;

/**
 * @author Hung Nguyen (nqhungvn@users.sourceforge.net)
 * @since Sept 26, 2005
 * @version $Id$
 */
public class UserActivitiesLogPlugin extends BaseComponentPlugin implements CacheListener {  
  private FIFOExoCache userActivitiesLogCache_ ;  
  private HibernateService hservice_ ;
  private CacheService cservice_ ;
  
  public UserActivitiesLogPlugin (HibernateService hservice,
                                  CacheService cservice) throws Exception{
    hservice_ = hservice ;
    cservice_ = cservice ;
    userActivitiesLogCache_ = (FIFOExoCache)cservice_.getCacheInstance("UserActivitiesLogCache") ;
    userActivitiesLogCache_.addCacheListener(this) ;   
  }   
  
  public void logUserActivities(String sessionId, String userName, String portalOwner, 
      long startTime, String accessIp) {
    try{
      UserActivitiesLogDataImpl impl = new UserActivitiesLogDataImpl() ;
      impl.logUserActivities(sessionId, userName, portalOwner, startTime, accessIp);      
      userActivitiesLogCache_.put(sessionId,impl) ;
    }catch (Exception ex){
      ex.printStackTrace() ;
    }    
  }
  
  public void updateLogUserActivities(String sessionId, long currentTime) {
    try {
      UserActivitiesLogDataImpl impl = 
        (UserActivitiesLogDataImpl) userActivitiesLogCache_.get(sessionId) ;
      if(impl == null) {
        impl = new UserActivitiesLogDataImpl() ;
        impl.setSessionId(sessionId) ;
        impl.setAccessCounter(1) ;
        impl.setStartTime(currentTime) ;
        impl.update(currentTime);        
        userActivitiesLogCache_.put(sessionId,impl) ;
      }else{
        impl.update(currentTime) ;
      }   
    } catch(Exception ex) {
      ex.printStackTrace() ;
    }
  }
  
  synchronized public void saveUserActivitiesLogData(UserActivitiesLogData data) throws Exception {
    Session session = hservice_.openSession() ;
    UserActivitiesLogDataImpl impl = 
      (UserActivitiesLogDataImpl) session.get(UserActivitiesLogDataImpl.class, data.getSessionId()) ;
    if(impl == null) {
      impl = (UserActivitiesLogDataImpl) data ;
      session.save(impl) ;
      session.flush() ;
    } else {
      impl.merge((UserActivitiesLogDataImpl)data) ;
      session.update(impl) ;
      session.flush() ;
    }
  }
  
  public PageList getUserLogDatas(UserLogQuery q) throws Exception {
    ObjectQuery oq = new ObjectQuery(UserActivitiesLogDataImpl.class);
    if(q.getMaxmin().equals("max")){
      oq.setDescOrderBy("duration") ;
    }else {
      oq.setAscOrderBy("duration") ;
    }         
    oq.addGT("startTime",  q.getFromDateTime()) ;
    oq.addLT("startTime",  q.getToDateTime()) ;
    if(q.getAccessIp().length() > 0)
      oq.addEQ("accessIp", q.getAccessIp()) ;  
    if(q.getUserName() != "")
      oq.addLIKE("userName", q.getUserName()) ;
    if(q.getPortalOwner() != "")
      oq.addLIKE("portalOwner", q.getPortalOwner()) ;
    if(q.getOrder() != "" && q.getOrder().equals("Desc")){
      oq.setDescOrderBy("startTime") ;
    }else if(q.getOrder() != "" && q.getOrder().equals("Asc")){
      oq.setAscOrderBy("startTime") ;
    }
    if(q.getDurationOrder() != "" && q.getDurationOrder().equals("Desc")){
      oq.setDescOrderBy("duration") ;
    }else if(q.getDurationOrder() != "" && q.getDurationOrder().equals("Asc")){
      oq.setAscOrderBy("duration") ;
    }
    return new DBObjectPageList(hservice_, oq);    
  }
  
  public Collection getUserLogChartDatas(UserLogQuery q, List style) throws Exception {
    Collection chartCollection = new ArrayList() ;
    for(int i=0; i < style.size(); i++){
      SelectItem chartType = (SelectItem)style.get(i) ; 
      ObjectQuery oq = new ObjectQuery(UserActivitiesLogDataImpl.class);
      oq.addSelect(chartType.value_) ;
      if(q.getMaxmin().equals("") || q.getMaxmin().length() == 0){
        oq.addSUM("accessCounter") ;
        oq.addSUM("duration") ;
        oq.setGroupBy(chartType.value_) ;
      }else {
        oq.addSelect("accessCounter") ;
        oq.addSelect("duration") ;
        if(q.getMaxmin().equals("max")){
          oq.setDescOrderBy("duration") ;
        }else {
          oq.setAscOrderBy("duration") ;
        }                
      }          
      oq.addGT("startTime",  q.getFromDateTime()) ;
      oq.addLT("startTime",  q.getToDateTime()) ;
      if(q.getUserName() != "")
        oq.addLIKE("userName", q.getUserName()) ;
      if(q.getPortalOwner() != "")
        oq.addLIKE("portalOwner", q.getPortalOwner()) ;
      if(q.getAccessIp().length() > 0)
        oq.addEQ("accessIp", q.getAccessIp()) ;      
      String groupQuery = oq.getHibernateGroupByQuery() ;
      Session session = hservice_.openSession() ; 
      chartCollection.add(session.createQuery(groupQuery.toString()).list());      
    }
    return chartCollection;    
  }
  
  public void  saveMonitorData() throws Exception {
    List cacheObjectList = userActivitiesLogCache_.getCachedObjects() ;
    for(int i = 0; i < cacheObjectList.size(); i++){
      UserActivitiesLogDataImpl data = (UserActivitiesLogDataImpl)cacheObjectList.get(i) ;
      saveUserActivitiesLogData(data) ;
    }
    userActivitiesLogCache_.clearCache() ;
  }
  
  public void onExpire(ExoCache cache, Serializable key, Object value) throws Exception {
    UserActivitiesLogData data = (UserActivitiesLogData) value ;
    saveUserActivitiesLogData(data) ;
  }
  
  public void onRemove(ExoCache arg0, Serializable arg1, Object arg2) throws Exception {    
  }
  public void onPut(ExoCache arg0, Serializable arg1, Object arg2) throws Exception {
  }
  public void onGet(ExoCache arg0, Serializable arg1, Object arg2) throws Exception {
  }
  public void onClearCache(ExoCache arg0) throws Exception {
  }
}
