/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.ecm.user;

import condition.* ;
import suites.share.* ;
import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;
/**
 * This suite does :
 *    1)Go to current user
 *    2)Add new folder and go to this folder
 *    3)Add new action
 *    4)Add new document which has name is approve
 *    5)Add new document which has name is disapprove
 *    6)logout and login with account exoadmin
 *    7)Go to workflow 
 *    8)Go to task controler to test actions approve and disapprove
 *    9)Delete created action
 *    
 * Created by The eXo Platform SARL 
 * Author : Dung Hoang
 *          dunghoang@exoplatform.com, jobdung@yahoo.com
 *          Hoa Nguyen Thi
 *          hoa2501@yahoo.com
 * Dec 27, 2005
 */
public class UserManageActionsSuite extends WebUnitSuite {
  static String DESC = "Go to Ecm Portlet User page, click on Info tab, click on Manage Action button and to do somethings" ;
  static String NAME = "UserManageActionsSuite" ;
  String COMMENT  = "You must run AdminViewSuite before run this suite";
  
  public UserManageActionsSuite() {    
    super(NAME, DESC) ;    
    addWebSuite(new UserAddFolderSuite());
    //-----Upload a file to created Folder
    addWebUnit(
      new ClickURLLinkUnit("UploadFileToFolder", "click on Upload button").
      setURL("&uicomponent=UIToolbar&op=upload").                  
      addValidator(new ExpectFormIdValidator("uploadForm"))      
    ) ;
    //----------Save File
    def is;
    is= Thread.currentThread().
        getContextClassLoader().
        getResourceAsStream("org/exoplatform/test/web/Util.class") ;    
    addWebUnit(
      new SubmitFormUnit("SaveFile", "click on save button").
      setFormId("uploadForm").   
      setFileField("upload","Unit.class", is).
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/home/users/#{client.name}/test-folder/upload")).
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIDocumentInfo&op=editDocument&objectId=/home/users/#{client.name}/test-folder/upload"))      
    ) ;
    //-----Select Admin tab
    addWebUnit(
      new ClickURLLinkUnit("SelectAdminTab", "click on Admin tab").
      setURL("&uicomponent=UIViewController&op=selectNode&objectId=Admin").             
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=manageVersions")).      
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=manageCategories")).
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=manageActions")).      
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=exportNode")).
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=importNode"))
    ) ;    
    //-----Manage Actions
    addWebUnit(
      new ClickURLLinkUnit("ClickOnManageActions", "click on Manage Actions button").
      setURL("&uicomponent=UIToolbar&op=manageActions").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIActionList&op=add"))      
    ) ;   
    //----------Add/Edit Actions
    addWebUnit(
      new ClickURLLinkUnit("AddAction", "click on add button").
      setURL("&uicomponent=UIActionList&op=add").   
      addValidator(new ExpectFormIdValidator("selectActionForm")).
      addValidator(new ExpectFormIdValidator("workflowForm"))            
    ) ;
    addWebUnit(
      new SubmitFormUnit("SelectActionOfType", "select action of type is exo:workflowAction").
      setFormId("selectActionForm").
      setField("actionType","exo:workflowAction").
      setField("op","change").  
      addValidator(new ExpectTextValidator("ContentValidation"))      
    ) ;
    //---------------Search Validator
    addWebUnit(
      new SubmitFormUnit("SearchValidator:", "click on search button").
      setFormId("workflowForm").   
      setField("op","selectValidator").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIExoPermissionBrowser&op=selectGroup&objectId=/admin"))      
    ) ;        
    addWebUnit(
      new ClickURLLinkUnit("SelectGroup", "select a group").
      setURL("&uicomponent=UIExoPermissionBrowser&op=selectGroup&objectId=/admin").                  
      addValidator(new ExpectFormIdValidator("workflowForm"))      
    ) ;
    //----------Save Action
    addWebUnit(
      new SubmitFormUnit("SaveNewAction", "select action of type, fill all informations and click save button").
      setFormId("workflowForm"). 
      setField("name","action-test").
      setField("lifecycle","add").      
      setField("description","description-test").
      setField("validator","*:/admin").
      setField("destWorkspace","production").                
      setField("destPath","/cms/publications").
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIActionList&op=view&objectId=action-test"))
    ) ;      
    //-----Save session
    addWebUnit(
      new ClickURLLinkUnit("SavedSession", "click on save Session button").
      setURL("&uicomponent=MainToolbar&op=saveSession").        
      addValidator(new ExpectTextValidator("All the changes have been saved"))      
    ) ;
    
    //Select Actions tab
    addWebUnit(
      new ClickURLLinkUnit("SelectActiontab", "Select Actions tab").
      setURL("&uicomponent=UIViewController&op=selectNode&objectId=Actions").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=addFolder"))      
    ) ;
    //Click on Add document
    addWebUnit(
      new ClickURLLinkUnit("AddDocument", "click on Add Document button").
      setURL("uicomponent=UIToolbar&op=addDocument").    
      addValidator(new ExpectFormIdValidator("selectDocumentForm")).
      addValidator(new ExpectFormIdValidator("documentForm"))
    ) ;
    //-----Add new Nt:file document
    addWebUnit(
      new SubmitFormUnit("SelectDocumentType", "select document type is nt:file").
      setFormId("selectDocumentForm").   
      setField("documentType","nt:file").
      setField("op","changeDocument").
      addValidator(new ExpectNoTextValidator("Image to upload")).
      addValidator(new ExpectTextValidator("File")).
      addValidator(new ExpectTextValidator("Content:"))
    ) ;
    //----------Save New Document
    addWebUnit(
      new SubmitFormUnit("SaveNewDocument", "click on save button").
      setFormId("documentForm").   
      setField("docName","approve-doc").
      setField("summary","content-approve").      
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIDocumentInfo&op=editDocument&objectId=/home/users/#{client.name}/test-folder/approve-doc"))      
    ) ;
    //Save session
    addWebUnit(
      new ClickURLLinkUnit("SavedSession", "click on save Session button").
      setURL("&uicomponent=MainToolbar&op=saveSession").        
      addValidator(new ExpectTextValidator("All the changes have been saved"))      
    ) ; 
    //Click on Add document
    addWebUnit(
      new ClickURLLinkUnit("AddDocument", "click on Add Document button").
      setURL("uicomponent=UIToolbar&op=addDocument").    
      addValidator(new ExpectFormIdValidator("selectDocumentForm")).
      addValidator(new ExpectFormIdValidator("documentForm"))
    ) ;
    //-----Add new Nt:file document
    addWebUnit(
      new SubmitFormUnit("SelectDocumentType", "select document type is nt:file").
      setFormId("selectDocumentForm").   
      setField("documentType","nt:file").
      setField("op","changeDocument").
      addValidator(new ExpectNoTextValidator("Image to upload")).
      addValidator(new ExpectTextValidator("File")).
      addValidator(new ExpectTextValidator("Content:"))
    ) ;
    //----------Save New Document
    addWebUnit(
      new SubmitFormUnit("SaveNewDocument", "click on save button").
      setFormId("documentForm").   
      setField("docName","disapprove-doc").
      setField("summary","content-disapprove").      
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIDocumentInfo&op=editDocument&objectId=/home/users/#{client.name}/test-folder/disapprove-doc"))
    ) ;
    //Save session
    addWebUnit(
      new ClickURLLinkUnit("SavedSession", "click on save Session button").
      setURL("&uicomponent=MainToolbar&op=saveSession").        
      addValidator(new ExpectTextValidator("All the changes have been saved"))      
    ) ;  
//    //-----Login with admin account
//    addWebUnit(
//      new ContextManagementUnit(ContextManagementUnit.SAVE, "Save Context")
//    );     
//    String[] roles = ["user", "admin"] ;
//    addWebSuite(new UserLoginSuite("exoadmin", "exo", roles)) ;   
//    //----------Goto Workflow and to do something
//    goToPage("/Site Map/Workflow") ;
//    addWebUnit(
//      new ClickURLLinkUnit("ClickOnTaskController", "click on task controller button").
//      setURL("&uicomponent=null&op=selectNode&objectId=UITaskListController").  
//      addValidator(new ExpectTextValidator("Manage"))      
//    ) ;    
//    addWebUnit(
//      new ClickURLLinkUnit("ClickOnManage", "click on manage link").
//      setURL("&uicomponent=TemplateRenderer&op=manageState&objectId=").        
//      addValidator(new ExpectFormIdValidator("taskForm"))            
//    ) ;    
//    addWebUnit(
//        new SubmitFormUnit("ClickApproveIcon", "click on Approve icon to approve this action").
//        setFormId("taskForm").   
//        setField("op","approve").
//        addValidator(new ExpectLinkWithURLValidator("&uicomponent=TemplateRenderer&op=manageState&objectId="))
//    ) ; 
//    addWebUnit(
//        new ClickURLLinkUnit("ClickOnManage", "click on manage link").
//        setURL("&uicomponent=TemplateRenderer&op=manageState&objectId=").  
//        addValidator(new ExpectFormIdValidator("taskForm"))            
//    ) ;    
//    addWebUnit(
//        new SubmitFormUnit("ClickDisApproveIcon", "click on Disapprove icon to disapprove this action").
//        setFormId("taskForm").   
//        setField("op","disapprove").
//        addValidator(new ExpectLinkWithURLValidator("&uicomponent=TemplateRenderer&op=manageState&objectId=",0))
//    ) ;  
//    goToPage("/Home/ecm") ;
//    addWebUnit(
//        new ClickURLLinkUnit("ClickOnProduction", "click on Production button").
//        setURL("&uicomponent=UIWorkspaceBrowser&op=selectWorkspace&objectId=production").                  
//        addValidator(new ExpectLinkWithURLValidator("portal:isSecure=false&uicomponent=UIDocumentInfo&op=changeNode&objectId=/home"))      
//    ) ;
//    addWebUnit(
//        new ClickURLLinkUnit("ClickOnCms", "click on home cms").
//        setURL("uicomponent=UIDocumentInfo&op=changeNode&objectId=/cms").             
//        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/cms/publications"))           
//    ) ;
//    addWebUnit(
//        new ClickURLLinkUnit("ClickOnPublications", "click on publications link").
//        setURL("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/cms/publications").
//        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIDocumentInfo&op=removeNode&objectId=/cms/publications/approve-doc")).      
//        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIDocumentInfo&op=changeNode&objectId=home/disapprove-doc",0))
//    ) ;
//    addWebUnit(
//        new ClickURLLinkUnit("DeleteApprove-doc", "click on delete icon on row approve-doc").
//        setURL("uicomponent=UIDocumentInfo&op=removeNode&objectId=/cms/publications/approve-doc").   
//        addValidator(new ExpectTextValidator("The node /cms/publications/approve-doc has been removed."))      
//    ) ;
//    //Save session
//    addWebUnit(
//      new ClickURLLinkUnit("SavedSession", "click on save Session button").
//      setURL("&uicomponent=MainToolbar&op=saveSession").        
//      addValidator(new ExpectTextValidator("All the changes have been saved"))      
//    ) ;  
//    //-----Back to current user
//    addWebUnit(
//      new ContextManagementUnit(ContextManagementUnit.RESTORE, "Restore Context")
//    ); 
    //-----Select Admin tab
    addWebUnit(
      new ClickURLLinkUnit("SelectAdminTab", "click on Admin tab").
      setURL("&uicomponent=UIViewController&op=selectNode&objectId=Admin").             
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=manageVersions")).      
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=manageCategories")).
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=manageActions")).      
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=exportNode")).
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=importNode"))
    ) ; 
    //-----Manage Actions
    addWebUnit(
      new ClickURLLinkUnit("ClickOnManageActions", "click on Manage Actions button").
      setURL("&uicomponent=UIToolbar&op=manageActions").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIActionList&op=add"))      
    ) ;
    //Delete created Action        
    addWebUnit(
      new ClickURLLinkUnit("DeleteCreatedAction", "delete created action").
      setURL("&uicomponent=UIActionList&op=delete&objectId=action-test").         
      addValidator(new ExpectNoTextValidator("action-test"))      
    ) ;
    //Saved Session
    addWebUnit(
      new ClickURLLinkUnit("SavedSession", "click on save Session button").
      setURL("&uicomponent=MainToolbar&op=saveSession").        
      addValidator(new ExpectTextValidator("All the changes have been saved"))      
    ) ;    
    //Select Actions tab
    addWebUnit(
      new ClickURLLinkUnit("SelectActiontab", "Select Actions tab").
      setURL("&uicomponent=UIViewController&op=selectNode&objectId=Actions").                  
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIDocumentInfo&op=removeNode&objectId=/home/users/#{client.name}/test-folder/disapprove-doc"))      .
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIDocumentInfo&op=removeNode&objectId=/home/users/#{client.name}/test-folder/disapprove-doc"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("DeleteDisapprove-doc", "Delete created Disapprove-doc").
      setURL("uicomponent=UIDocumentInfo&op=removeNode&objectId=/home/users/#{client.name}/test-folder/disapprove-doc").     
      addValidator(new ExpectTextValidator("The node /home/users/#{client.name}/test-folder/disapprove-doc has been removed."))      
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("DeleteUploadedFile", "Delete uploaded file").
      setURL("&uicomponent=UIDocumentInfo&op=removeNode&objectId=/home/users/#{client.name}/test-folder/upload").     
      addValidator(new ExpectTextValidator("The node /home/users/#{client.name}/test-folder/upload has been removed."))      
    ) ; 
    //Select Workspace
    addWebUnit(
      new ClickURLLinkUnit("SelectWorkspace", "select Workspace button").
      setURL("&uicomponent=MainToolbar&op=selectWorkspace").                  
      addValidator(new ExpectTextValidator("Workspaces "))      
    ) ;   
  }                 
  public java.util.List getRequiredSuites() { 
    String[] roles = ["user"] ;
    return [new NewAccountSuite(), new UserLoginSuite("#{client.name}", "#{client.name}", roles)] ;
  }  
}