/**************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved. *
 * Please look at license.txt in info directory for more license detail.  *
 **************************************************************************/
package suites.portal;

import condition.* ;
import suites.share.* ;
import suites.share.UserLoginSuite;
import suites.share.UserLogoutSuite;

import org.exoplatform.test.*;
import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;
/**
 * This suite does :
 *    1)Create a new page with view and edit permission is any guest  
 *    2)Check view for portlet on created newPage
 *      2.1)Check Show Info Bar
 *      2.1)Check Show Portlet Mode
 *      2.2)Check Show Window State
 *    3)Check permission for  portlet on created newPage
 *      3.1)Check permission portlet is owner
 *      3.2)Check permission portlet is any admin
 *      3.3)Check permission portlet is any user
 *           
 * Created by The eXo Platform SARL
 * Author : Hoa  Nguyen Thi
 *          hoa2501@yahoo.com
 * Mar 31, 2006
 */
public class ExoPortalPermPortletSuite extends WebUnitSuite {
  String COMMENT  = "You must run PortletRegistrySuite suite before run this suite";
  String[] roles=["user"];
  public ExoPortalPermPortletSuite() {
    super("ExoPortalPermPortletSuite", "Test some permission for add new node of sitemap") ;
    addWebUnit( CommonUnit.CLICK_EDIT_NAVIGATION_MODE) ;
    //Go to Browse Page
    addWebUnit(CommonUnit.CLICK_BROWSE_PAGES_NAV) ;
    //-----Add new page
    addWebSuite(new AddPageBrowseSuite("exoportal","web","NavigationPortlet","*:/guest","*:/guest")) ;
    //----------Set show info bar for navigation portlet amd Check show info bar  
    addWebSuite(new EditNavPortletSuite("exoportal","newPage",false,true,true,"*:/user",
        "windowState=maximized",0));
    //----------Set show portlet mode for navigation portlet and Check show portlet mode
    addWebSuite(new EditNavPortletSuite("exoportal","newPage",true,false,true,"*:/user",
        "portal:componentId=NavigationPortlet&portal:portletMode=edit",0));
    //----------Set show window state for navigation portlet and Check show window state
    addWebSuite(new EditNavPortletSuite("exoportal","newPage",true,true,false,"*:/user",
        "windowState=maximized",0));
   
    //Check permission portlet is owner   
    addWebSuite(new EditNavPortletSuite("exoportal","newPage",true,true,true,"owner:/portal",
                                        "windowState=maximized",-1));   
    //-----Login with demo account
    addWebUnit(
      new ContextManagementUnit(ContextManagementUnit.SAVE, "Save Context")
    ); 
    String[] roles=["user"];
    addWebSuite(new UserLoginSuite("demo", "exo", roles)) ;   
    //**-----Add a new node
    addWebUnit( CommonUnit.CLICK_EDIT_NAVIGATION_MODE) ;
    addWebUnit( CommonUnit.CLICK_SHOW_NAV) ;
    addWebUnit(
        new ClickURLLinkUnit("DeleteExistedChildNode", "Delete if existed ChildNode").
        setURL("uicomponent=UINavigationManager&op=deleteNode&objectId=/ChildNode").
        addCondition(new HaveLinkWithURLCondition("uicomponent=UINavigationManager&op=deleteNode&objectId=/ChildNode")).
        addValidator(new ExpectNoTextValidator("ChildNode"))
    ) ;  
    //-----Create new node on HomePage with permission is any user
    addWebUnit(
      new ClickURLLinkUnit("ClickAddNode(onHomePage)", "Click 'add' on homepage").
      setURL("uicomponent=UINavigationManager&op=addNode&objectId=/").
      addValidator(new ExpectFormIdValidator("pageNodeForm")).
      addValidator(new ExpectTextValidator("Add/Edit Node")) 
    ) ; 
    addWebSuite(new SearchPermissionSuite("pageNodeForm","View",
       "PhaseId[1].searchGroupForViewPerm","*","guest"));    
    addWebSuite(new SearchPermissionSuite("pageNodeForm","Edit",
                     "PhaseId[1].searchGroupForEditPerm", "*" ,"guest"));
    addWebUnit(
        new SubmitFormUnit("SearchPageReference", "Click Search Page Reference icon").
        setFormId("pageNodeForm").
        setField("op","PhaseId[1].searchXhtmlRefPage").
        addValidator(new ExpectFormIdValidator("searchForm"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("SearchPageOfExo", "Click Search icon").
        setFormId("searchForm").
        setField("owner","exoportal").
        setField("op","searchPages").
        addValidator(new ExpectLinkWithURLValidator("?uicomponent=UIPageList&op=select&objectId=exoportal:newPage"))
    ) ;   
    addWebUnit(
        new ClickURLLinkUnit("SelectPageReference", "Select Page Reference").
        setURL("uicomponent=UIPageList&op=select&objectId=exoportal:newPage").
        addValidator(new ExpectFormIdValidator("pageNodeForm"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("Create'ChildNode'", "Create a child node with some properties are default").
      setFormId("pageNodeForm").
      setField("name","ChildNode").
      setField("displayName","ChildNode").
      setField("op","save").
      addValidator(new ExpectLinkWithTextValidator("ChildNode"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("SaveChildNode", "Click Save icon").
        setURL("&uicomponent=UIToolbarNav&op=saveNode").
        addValidator(new ExpectTextValidator("ChildNode"))
    ) ;   
    addWebUnit(
        new ClickLinkWithText("ClickChildNodeButton", "Click on ChildNode button of navigation ").
        setTextLink("ChildNode").
        addValidator(new ExpectLinkWithURLValidator("portletMode=edit",0))
    ) ;   
    //-----Back to current user
    addWebUnit(
      new ContextManagementUnit(ContextManagementUnit.RESTORE, "Restore Context")
    ); 
       
    //Check permissin portlet is any admin   
    addWebSuite(new EditNavPortletSuite("exoportal","newPage",true,true,true,"*:/admin",
        "windowState=maximized",-1));
    //-----Login with admin account
    addWebUnit(
      new ContextManagementUnit(ContextManagementUnit.SAVE, "Save Context")
    ); 
    roles = ["user", "admin"] ;
    addWebSuite(new UserLoginSuite(CommonUnit.NAME_EXO_ADMIN, "exo", roles));
    //**-----Add a new node
    addWebUnit( CommonUnit.CLICK_EDIT_NAVIGATION_MODE) ;
    addWebUnit( CommonUnit.CLICK_SHOW_NAV) ;
    addWebUnit(
        new ClickURLLinkUnit("DeleteExistedChildNode", "Delete if existed ChildNode").
        setURL("uicomponent=UINavigationManager&op=deleteNode&objectId=/ChildNode").
        addCondition(new HaveLinkWithURLCondition("uicomponent=UINavigationManager&op=deleteNode&objectId=/ChildNode")).
        addValidator(new ExpectNoTextValidator("ChildNode"))
    ) ;  
    //-----Create new node on HomePage with permission is any guest
    addWebUnit(
      new ClickURLLinkUnit("ClickAddNode(onHomePage)", "Click 'add' on homepage").
      setURL("uicomponent=UINavigationManager&op=addNode&objectId=/").
      addValidator(new ExpectFormIdValidator("pageNodeForm")).
      addValidator(new ExpectTextValidator("Add/Edit Node")) 
    ) ; 
    addWebSuite(new SearchPermissionSuite("pageNodeForm","View",
       "PhaseId[1].searchGroupForViewPerm","*","guest"));    
    addWebSuite(new SearchPermissionSuite("pageNodeForm","Edit",
                     "PhaseId[1].searchGroupForEditPerm", "*" ,"guest"));
    addWebUnit(
        new SubmitFormUnit("SearchPageReference", "Click Search Page Reference icon").
        setFormId("pageNodeForm").
        setField("op","PhaseId[1].searchXhtmlRefPage").
        addValidator(new ExpectFormIdValidator("searchForm"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("SearchPageOfExo", "Click Search icon").
        setFormId("searchForm").
        setField("owner","exoportal").
        setField("op","searchPages").
        addValidator(new ExpectLinkWithURLValidator("?uicomponent=UIPageList&op=select&objectId=exoportal:newPage"))
    ) ;   
    addWebUnit(
        new ClickURLLinkUnit("SelectPageReference", "Select Page Reference").
        setURL("uicomponent=UIPageList&op=select&objectId=exoportal:newPage").
        addValidator(new ExpectFormIdValidator("pageNodeForm"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("Create'ChildNode'", "Create a child node with some properties are default").
      setFormId("pageNodeForm").
      setField("name","ChildNode").
      setField("displayName","ChildNode").
      setField("op","save").
      addValidator(new ExpectLinkWithTextValidator("ChildNode"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("SaveChildNode", "Click Save icon").
        setURL("&uicomponent=UIToolbarNav&op=saveNode").
        addValidator(new ExpectTextValidator("ChildNode"))
    ) ;   
    addWebUnit(
        new ClickLinkWithText("ClickChildNodeButton", "Click on ChildNode button of navigation ").
        setTextLink("ChildNode").
        addValidator(new ExpectLinkWithURLValidator("portletMode=edit"))
    ) ;   
    addWebUnit( CommonUnit.CLICK_SHOW_NAV) ;
    addWebUnit(
        new ClickURLLinkUnit("DeleteChildNode", "Delete created ChildNode").
        setURL("uicomponent=UINavigationManager&op=deleteNode&objectId=/ChildNode").
        addValidator(new ExpectNoTextValidator("ChildNode"))
    ) ;    
    //-----Back to current user
    addWebUnit(
      new ContextManagementUnit(ContextManagementUnit.RESTORE, "Restore Context")
    ); 
       
    //Check view and edit permission of childNode is any user
    addWebSuite(new EditNavPortletSuite("exoportal","newPage",true,true,true,"*:/user",
        "windowState=maximized",-1));
    //-----Login with demo account
    addWebUnit(
      new ContextManagementUnit(ContextManagementUnit.SAVE, "Save Context")
    );
    addWebSuite(new UserLoginSuite("demo", "exo", roles)) ; 
    addWebUnit( CommonUnit.CLICK_EDIT_NAVIGATION_MODE) ;
    addWebUnit( CommonUnit.CLICK_SHOW_NAV) ;
    addWebUnit(
        new ClickURLLinkUnit("DeleteExistedChildNode", "Delete if existed ChildNode").
        setURL("uicomponent=UINavigationManager&op=deleteNode&objectId=/ChildNode").
        addCondition(new HaveLinkWithURLCondition("uicomponent=UINavigationManager&op=deleteNode&objectId=/ChildNode")).
        addValidator(new ExpectNoTextValidator("ChildNode"))
    ) ;  
    addWebUnit(
        new ClickURLLinkUnit("ClickAddNode(onHomePage)", "Click 'add' on homepage").
        setURL("uicomponent=UINavigationManager&op=addNode&objectId=/").
        addValidator(new ExpectFormIdValidator("pageNodeForm")).
        addValidator(new ExpectTextValidator("Add/Edit Node")) 
      ) ; 
    addWebSuite(new SearchPermissionSuite("pageNodeForm","View",
         "PhaseId[1].searchGroupForViewPerm","*","guest"));    
    addWebSuite(new SearchPermissionSuite("pageNodeForm","Edit",
                       "PhaseId[1].searchGroupForEditPerm", "*" ,"guest"));
    addWebUnit(
        new SubmitFormUnit("SearchPageReference", "Click Search Page Reference icon").
        setFormId("pageNodeForm").
        setField("op","PhaseId[1].searchXhtmlRefPage").
        addValidator(new ExpectFormIdValidator("searchForm"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("SearchPageOfExo", "Click Search icon").
        setFormId("searchForm").
        setField("owner","exoportal").
        setField("op","searchPages").
        addValidator(new ExpectLinkWithURLValidator("?uicomponent=UIPageList&op=select&objectId=exoportal:newPage"))
    ) ;   
    addWebUnit(
        new ClickURLLinkUnit("SelectPageReference", "Select Page Reference").
        setURL("uicomponent=UIPageList&op=select&objectId=exoportal:newPage").
        addValidator(new ExpectFormIdValidator("pageNodeForm"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("Create'ChildNode'", "Create ChildNode with some properties").
      setFormId("pageNodeForm").
      setField("name","ChildNode").
      setField("displayName","ChildNode").
      setField("op","save").
      addValidator(new ExpectLinkWithTextValidator("ChildNode"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("SaveChildNode", "Click Save icon").
        setURL("&uicomponent=UIToolbarNav&op=saveNode").
        addValidator(new ExpectTextValidator("ChildNode"))
    ) ;   
    addWebUnit(
        new ClickLinkWithText("ClickChildNodeButton", "Click on ChildNode button of navigation ").
        setTextLink("ChildNode").
        addValidator(new ExpectLinkWithURLValidator("portletMode=edit"))
    ) ;   
    addWebUnit( CommonUnit.CLICK_SHOW_NAV) ;
    addWebUnit(
        new ClickURLLinkUnit("DeleteChildNode", "Delete created ChildNode").
        setURL("uicomponent=UINavigationManager&op=deleteNode&objectId=/ChildNode").
        addValidator(new ExpectNoTextValidator("ChildNode"))
    ) ;    
    //-----Back to current user
    addWebUnit(
      new ContextManagementUnit(ContextManagementUnit.RESTORE, "Restore Context")
    ); 
    //-----Delete  existed newpage
    addWebUnit(
        new ClickURLLinkUnit("DeleteNewPage", "Delete existed NewPage").
        setURL("uicomponent=UIPageList&op=delete&objectId=exoportal:newPage").
        addValidator(new ExpectTextValidator("The page has been deleted successfully")) 
    ) ; 
    addWebUnit(
        new ClickURLLinkUnit("ClickOnSave", "Click on save icon").
        setURL("uicomponent=UIToolbarNav&op=saveNode").
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIToolbarNav&op=saveNode"))
    ) ;
  }  
  public java.util.List getRequiredSuites() {         
    String[] roles = ["user"] ;    
    return [new UserLoginSuite("exoportal", "exo", roles)] ;  
  } 
}
